#!/bin/sh
#   Copyright (c) 2010 digitalSTROM.org, Zurich, Switzerland
#
#   Author: Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>
#
#   This file is part of digitalSTROM Server.
#
#   netinfo.sh is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   netinfo.sh is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this file. If not, see <http://www.gnu.org/licenses/>.

# source common functions
. ./errlib.sh
. ./jsonlib.sh
. ./util.sh

NTPCONFIG="/etc/default/ntpclient"
ZONEINFO="/usr/share/zoneinfo/"

set_ntp() {
    if [[ -z "$1" ]]; then
        return $ERR_OK
    fi

    ENABLED=
    NTPSERVER=
    NTPINTERVAL=600
    if [[ -f $NTPCONFIG ]]; then
        . $NTPCONFIG
    fi

    if [ -x /etc/init.d/ntpd ]; then
        /etc/init.d/ntpd stop 1>&2
    fi

    if [[ "$ENABLED" != "$1" ]]; then
        if [[ -n "$2" ]]; then
            NTPSERVER="$2"
        fi
        echo "# enable or disable NTP client" > $NTPCONFIG
        echo "ENABLED=$1" >> $NTPCONFIG
        echo >> $NTPCONFIG
        echo "# seconds between time checks" >> $NTPCONFIG
        echo "NTPINTERVAL=$NTPINTERVAL" >> $NTPCONFIG
        echo >> $NTPCONFIG
    fi

    if [[ "$1" == "yes" ]]; then
        if [ -x /etc/init.d/ntpd ]; then
            /etc/init.d/ntpd start 1>&2
        fi
    fi

    return $ERR_OK
}

set_date() {
    DAY=`echo $1 | cut -d - -f1`
    MONTH=`echo $1 | cut -d - -f2`
    YEAR=`echo $1 | cut -d - -f3`

    echo "$DAY" | grep "[^0-9]" > /dev/null 2>&1
    if [ "$?" -eq "0" ]; then
        json_error $ERR_DATE "Invalid day given!"
        return $ERR_DATE
    fi

    echo "$MONTH" | grep "[^0-9]" > /dev/null 2>&1
    if [ "$?" -eq "0" ]; then
        json_error $ERR_DATE "Invalid month given!"
        return $ERR_DATE
    fi

    echo "$YEAR" | grep "[^0-9]" > /dev/null 2>&1
    if [ "$?" -eq "0" ]; then
        json_error $ERR_DATE "Invalid year given!"
        return $ERR_DATE
    fi
    
    if [[ -n "$DAY" ]]; then
        if [[ $DAY -lt 1 ]] || [[ $DAY -gt 31 ]]; then
            json_error $ERR_DATE "Invalid day given!"
            return $ERR_DATE
        fi
    else
        json_error $ERR_DATE "Could not parse day!"
        return $ERR_DATE
    fi

    LENGTH=`expr length $DAY`
    if [[ $LENGTH -lt 2 ]]; then
        DAY="0$DAY"
    fi

    if [[ -n "$MONTH" ]]; then
        if [[ $MONTH -lt 1 ]] || [[ $MONTH -gt 12 ]]; then
            json_error $ERR_DATE "Invalid month given!"
            return $ERR_DATE
        fi
    else
        json_error $ERR_DATE "Could not parse month!"
        return $ERR_DATE
    fi

    LENGTH=`expr length $MONTH`
    if [[ $LENGTH -lt 2 ]]; then
        MONTH="0$MONTH"
    fi

    if [[ -n "$YEAR" ]]; then
        LENGTH=`expr length $YEAR`
        if [[ $LENGTH -lt 4 ]]; then
            json_error $ERR_DATE "Year must consist of four digits!"
            return $ERR_DATE
        fi
    else
        json_error $ERR_DATE "No year given!"
        return $ERR_DATE
    fi

# you can not set the date alone, you have to set the time as well
    HOUR=`date +%H`
    MINUTE=`date +%M`
    SECOND=`date +%S`

    date -s "$YEAR-$MONTH-$DAY $HOUR:$MINUTE:$SECOND" 1>&2
    if [[ "$?" -ne "0" ]]; then
        json_error $ERR_DATE "Error setting date!"
        return $ERR_DATE
    fi

    return $ERR_OK
}

set_time() {
    HOURS=`echo $1 | cut -d : -f1`
    MINUTES=`echo $1 | cut -d : -f2`
    SECONDS=`echo $1 | cut -d : -f3`

    echo "$HOURS" | grep "[^0-9]" > /dev/null 2>&1
    if [ "$?" -eq "0" ]; then
        json_error $ERR_DATE "Invalid hours given!"
        return $ERR_TIME
    fi

    echo "$MINUTES" | grep "[^0-9]" > /dev/null 2>&1
    if [ "$?" -eq "0" ]; then
        json_error $ERR_TIME "Invalid minutes given!"
        return $ERR_TIME
    fi

    if [[ -n "$SECONDS" ]]; then
        echo "$SECONDS" | grep "[^0-9]" > /dev/null 2>&1
        if [ "$?" -eq "0" ]; then
            json_error $ERR_TIME "Invalid seconds given!"
            return $ERR_TIME
        fi
    fi
    
    if [[ -n "$HOURS" ]]; then
        if [[ $HOURS -lt 0 ]] || [[ $HOURS -gt 23 ]]; then
            json_error $ERR_TIME "Invalid hours given!"
            return $ERR_TIME
        fi
    else
        json_error $ERR_TIME "Could not parse hours!"
        return $ERR_TIME
    fi

    if [[ -n "$MINUTES" ]]; then
        if [[ $MINUTES -lt 0 ]] || [[ $MINUTES -gt 59 ]]; then
            json_error $ERR_TIME "Invalid minutes given!"
            return $ERR_TIME
        fi
    else
        json_error $ERR_TIME "Could not parse minutes!"
        return $ERR_TIME
    fi

    if [[ -n "$SECONDS" ]]; then
        if [[ $SECONDS -lt 0 ]] || [[ $SECONDS -gt 59 ]]; then
            json_error $ERR_TIME "Invalid seconds given!"
            return $ERR_TIME
        fi
    fi

    TIMESTR="$HOURS:$MINUTES"

    if [[ -n "$SECONDS" ]]; then
        TIMESTR="$TIMESTR:$SECONDS"
    fi


    date -s "$TIMESTR" 1>&2
    if [[ "$?" -ne "0" ]]; then
        json_error $ERR_TIME "Error setting time!"
        return $ERR_TIME
    fi

    /sbin/hwclock -f /dev/rtc1 --systohc 1>&2

    return $ERR_OK
}

echo "Content-Type: application/json"
echo ""

DATE=
TIME=
TIMEZONE=

# "$FORM_ntp" "$FORM_settime" "$FORM_setdate" "$FORM_timezone"

if [[ -n "$4" ]]; then
    set_timezone "$4"
    RETCODE="$?"                                                                
    if [[ "$RETCODE" -ne "$ERR_OK" ]]; then
        if [[ $RETCODE -eq $ERR_NOZONES ]]; then
            json_error $ERR_NOZONES "Requested time zone is not installed!"
        elif [[ $RETCODE -eq $ERR_LINK ]]; then
            json_error $ERR_LINK "Could not set requested zone, link failed!"
        fi
        exit 1                                                            
    fi           
fi

if [[ "$1" == "on" ]]; then 
    set_ntp "yes"
    json_ok
    exit 0
elif [[ "$1" == "off" ]]; then
    set_ntp "no"
fi

if [[ -n "$3" ]]; then
    set_date "$3"
    RETCODE="$?"
    if [[ "$RETCODE" -ne "$ERR_OK" ]]; then
        json_error $RETCODE "Could not set date"
        exit 1
    fi
fi

if [[ -n "$2" ]]; then
    set_time "$2"
    RETCODE="$?"
    if [[ "$RETCODE" -ne "$ERR_OK" ]]; then
        json_error $RETCODE "Could not set time"
        exit 1
    fi
fi

json_ok
